@@ -25,6 +25,7 @@ from .. import message
 from .. import util
 from ..code_chunks import CodeChunk, CodeKey
 from ..code_collections import Session, Source
+from ..codebraid_defaults import CodebraidDefaults
 from ..progress import Progress
 from ..version import __version__ as codebraid_version
 from . import exec_builtin
@@ -46,9 +47,9 @@ class CodeProcessor(object):
                  cache_path: pathlib.Path,
                  cache_key: str,
                  origin_paths_for_cache: Optional[List[pathlib.Path]],
-                 code_defaults: Optional[Dict],
-                 session_defaults: Optional[Dict],
+                 codebraid_defaults: CodebraidDefaults,
                  only_code_output: bool,
+                 no_execute: bool,
                  progress: Progress):
         self.code_chunks = code_chunks
         self.cross_origin_sessions = cross_origin_sessions
@@ -61,10 +62,10 @@ class CodeProcessor(object):
             self._origin_paths_for_cache_as_strings = None
         else:
             self._origin_paths_for_cache_as_strings = [p.as_posix() for p in origin_paths_for_cache]
-        self.code_defaults = code_defaults
-        self.session_defaults = session_defaults
+        self.codebraid_defaults = codebraid_defaults
         self._only_code_output = only_code_output
         self._progress = progress
+        self._no_execute = no_execute
 
         self._old_cache_index: Optional[Dict] = None
         self._cache_key_path = cache_path / cache_key
@@ -72,13 +73,13 @@ class CodeProcessor(object):
         self._cache_lock_path = cache_path / cache_key / f'{cache_key}.lock'
 
         self._sessions: Dict[CodeKey, Session] = util.KeyDefaultDict(
-            lambda x: Session(x, code_defaults=self.code_defaults, session_defaults=self.session_defaults)
+            lambda x: Session(x, codebraid_defaults=self.codebraid_defaults)
         )
         self._session_hash_root_sets: Dict[str, Set] = collections.defaultdict(set)
         self._cached_sessions: Set[Session] = set()
         self._named_code_chunks: Dict[str, CodeChunk] = {}
 
-        self._sources: Dict[CodeKey, Source] = util.KeyDefaultDict(lambda x: Source(x))
+        self._sources: Dict[CodeKey, Source] = util.KeyDefaultDict(lambda x: Source(x, codebraid_defaults=self.codebraid_defaults))
 
         # Use `atexit` to improve the odds of cleanup in the event of an
         # unexpected exit.  `cleanup()` ends by invoking
@@ -330,7 +331,9 @@ class CodeProcessor(object):
         '''
         placeholder_lang_num = 0
         for cc in self.code_chunks:
-            if cc.execute:
+            if cc.execute or 'session' in cc.options:
+                # `cc.execute` covers valid commands, 'session' key covers
+                # invalid commands with named sessions
                 code_collection_name = cc.options['session']
                 code_collection_type = 'session'
             else:
@@ -341,7 +344,7 @@ class CodeProcessor(object):
             else:
                 origin_name = cc.origin_name
             cc.key = CodeKey(cc.options['lang'], code_collection_name, code_collection_type, origin_name)
-            if cc.options['inherited_lang']:
+            if cc.options['lang'] is None or cc.options['inherited_lang']:
                 cc.options['placeholder_lang'] = f'{placeholder_lang_num}'
                 placeholder_lang_num += 1
             if cc.execute:
@@ -496,6 +499,8 @@ class CodeProcessor(object):
             session_cache = saved_cache['cache'][session.hash]
         except KeyError:
             return False
+        if session_cache['session_status_prevent_exec'] and not self._no_execute:
+            return False
         for msg_name, msg_dict in session_cache['session_errors']:
             session.errors.append(message.message_name_to_class_map[msg_name](**msg_dict))
         for msg_name, msg_dict in session_cache['session_warnings']:
@@ -538,7 +543,7 @@ class CodeProcessor(object):
             'cache': {},
         }
         for session in self._session_hash_root_sets[update_session.hash_root]:
-            if session.status.prevent_caching:
+            if session.status.prevent_caching or (session.needs_exec and not session.did_exec):
                 continue
             session_code_chunks_cache = {}
             for index, chunk in enumerate(session.code_chunks):
@@ -559,6 +564,7 @@ class CodeProcessor(object):
                     # `str(index)` because keys for JSON cache are strings
                     session_code_chunks_cache[str(index)] = chunk_cache
             hash_root_cache['cache'][session.hash] = {
+                'session_status_prevent_exec': session.status.prevent_exec,
                 'session_errors': [(x.type, x.as_dict()) for x in session.errors if x.is_cacheable],
                 'session_warnings': [(x.type, x.as_dict()) for x in session.warnings if x.is_cacheable],
                 'session_files': session.files,
