@@ -54,11 +54,13 @@ async def exec(session: Session, *, cache_key_path: pathlib.Path, progress: Prog
                 commands = command_or_commands
             for command in commands:
                 if session.status.prevent_exec:
+                    progress.session_exec_stage_end(session, stage=stage)
                     break
                 subproc = SessionSubprocess(session, stage, command, temp_dir_path, progress)
                 await subproc.start()
                 await subproc.wait()
             progress.session_exec_stage_end(session, stage=stage)
+        progress.session_finished(session)
 
 
 class SessionSubprocess(object):
@@ -68,7 +70,7 @@ class SessionSubprocess(object):
         self.encoding = getattr(session.lang_def, f'{stage}_encoding')
         self.has_interpreter_script = session.lang_def.interpreter_script is not None
         template_dict = {
-            'executable': session.executable.as_posix(),
+            'executable': session.executable,
             'extension': session.lang_def.extension,
             'run_delim_start': session.run_delim_start,
             'run_delim_hash': session.run_delim_hash,
@@ -87,7 +89,17 @@ class SessionSubprocess(object):
                 'run_script': session.lang_def.interpreter_script.as_posix(),
                 'buffering': 'line',
             })
-        program_with_args = [s.format(**template_dict) for s in shlex.split(command)]
+        program_with_args: list[str] = []
+        for s in shlex.split(command):
+            if s == '{executable_opts}':
+                if session.executable_opts:
+                    program_with_args.extend(session.executable_opts)
+                continue
+            if s == '{args}':
+                if session.args:
+                    program_with_args.extend(session.args)
+                continue
+            program_with_args.append(s.format(**template_dict))
         if platform.system() == 'Windows':
             # Modify args since subprocess.Popen() ignores PATH
             #   * https://bugs.python.org/issue8557
@@ -246,7 +258,7 @@ class SessionSubprocess(object):
         output_str = self._decode(output, output_type=output_type, code_chunk=code_chunk)
         if output_type == 'stderr':
             output_str = self._sync_stderr_or_compile_output(output_str, code_chunk=code_chunk)
-        getattr(self.progress, f'chunk_{output_type}')(self.session, chunk=code_chunk, output=output_str)
+        getattr(self.progress, f'session_chunk_{output_type}')(self.session, chunk=code_chunk, output=output_str)
         getattr(code_chunk, f'{output_type}_lines').extend(util.splitlines_lf(output_str))
 
     def _run_line_number_to_origin(self, run_line_number: int) -> CodeLineOrigin | tuple[None, None]:
@@ -385,7 +397,7 @@ class SessionSubprocess(object):
     async def _sync_chunk_start_delims(self, code_chunk: CodeChunk):
         self._sync_chunk_start_delims_state[code_chunk.index] += 1
         if self._sync_chunk_start_delims_state[code_chunk.index] == 2:
-            self.progress.chunk_start(self.session, chunk=code_chunk)
+            self.progress.session_chunk_start_exec(self.session, chunk=code_chunk)
             return
         self._sync_chunk_start_delims_waiting += 1
         while self._sync_chunk_start_delims_state[code_chunk.index] < 2:
@@ -403,7 +415,7 @@ class SessionSubprocess(object):
     async def _sync_chunk_end_delims(self, code_chunk: CodeChunk):
         self._sync_chunk_end_delims_state[code_chunk.index] += 1
         if self._sync_chunk_end_delims_state[code_chunk.index] == 2:
-            self.progress.chunk_end(self.session, chunk=code_chunk)
+            self.progress.session_chunk_end_exec(self.session, chunk=code_chunk)
             return
         self._sync_chunk_end_delims_waiting += 1
         while self._sync_chunk_end_delims_state[code_chunk.index] < 2:
