@@ -12,10 +12,12 @@ from __future__ import annotations
 
 
 import base64
+import collections
+import pathlib
 import queue
 import re
+import subprocess
 import time
-import pathlib
 try:
     import jupyter_client
 except ImportError:
@@ -27,20 +29,41 @@ from ..progress import Progress
 
 
 _ansi_color_escape_code_re = re.compile('\x1b.*?m')
-
+_version_number_re = re.compile(r'(?P<major>\d+)(?:\.(?P<minor>\d+)(?:\.(?P<patch>\d+))?)?')
 
 kernel_name_aliases: dict[str, str] = {}
+kernel_name_collisions: dict[str, set[str]] = collections.defaultdict(set)
 if jupyter_client is not None:
-    duplicates = set()
     for k, v in jupyter_client.kernelspec.KernelSpecManager().get_all_specs().items():
         for alias in [k.lower(), v['spec']['display_name'].lower(), v['spec']['language'].lower()]:
             if alias in kernel_name_aliases:
-                duplicates.add(alias)
+                kernel_name_collisions[alias].add(k)
             else:
                 kernel_name_aliases[alias] = k
-    for k in duplicates:
-        del kernel_name_aliases[k]
-    del duplicates
+    for alias in kernel_name_collisions:
+        kernel_name_collisions[alias].add(kernel_name_aliases[alias])
+        if len(kernel_name_collisions[alias]) > 1:
+            name_to_version_map = {}
+            for k in kernel_name_collisions[alias]:
+                match = _version_number_re.search(k)
+                if match is None:
+                    name_to_version_map[k] = None
+                else:
+                    major = int(match.group('major'))
+                    minor = int(match.group('minor')) if match.group('minor') is not None else 0
+                    patch = int(match.group('patch')) if match.group('patch') is not None else 0
+                    name_to_version_map[k] = (major, minor, patch)
+            if not all(v is not None for v in name_to_version_map.values()):
+                del kernel_name_aliases[alias]
+            else:
+                last_version = None
+                for k, v in name_to_version_map.items():
+                    if last_version is None or v > last_version:
+                        kernel_name_aliases[alias] = k
+                        last_version = v
+                    elif v == last_version:
+                        del kernel_name_aliases[alias]
+                        break
 
 
 mime_type_to_file_extension_map: dict[str, str] = {
@@ -83,7 +106,13 @@ async def exec(session: Session, *, cache_key_path: pathlib.Path, progress: Prog
 
     kernel_name = kernel_name_aliases.get(session.jupyter_kernel.lower())
     if kernel_name is None:
-        msg = f'No Jupyter kernel was found for "{session.jupyter_kernel}"'
+        if session.jupyter_kernel in kernel_name_collisions:
+            msg = (
+                f'''Jupyter kernel "{session.jupyter_kernel}" is ambiguous; '''
+                f'''could refer to {', '.join(f'"{k}"' for k in kernel_name_collisions[session.jupyter_kernel])}'''
+            )
+        else:
+            msg = f'No Jupyter kernel was found for "{session.jupyter_kernel}"'
         session.errors.append(message.SysConfigError(msg))
         progress.session_exec_stage_end(session, stage='run')
         progress.session_finished(session)
@@ -91,7 +120,7 @@ async def exec(session: Session, *, cache_key_path: pathlib.Path, progress: Prog
 
     kernel_manager = jupyter_client.AsyncKernelManager(kernel_name=kernel_name)
     try:
-        await kernel_manager.start_kernel()
+        await kernel_manager.start_kernel(stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
     except jupyter_client.kernelspec.NoSuchKernel:
         msg = f'No Jupyter kernel was found for "{session.jupyter_kernel}"'
         session.errors.append(message.SysConfigError(msg))
@@ -135,11 +164,11 @@ async def exec(session: Session, *, cache_key_path: pathlib.Path, progress: Prog
                 incomplete_cc_stack.append(cc)
                 continue
             if not incomplete_cc_stack:
-                progress.session_chunk_start(session, chunk=cc)
+                progress.session_chunk_start_exec(session, chunk=cc)
                 cc_jupyter_id = kernel_client.execute(cc.code_str)
             else:
                 incomplete_cc_stack.append(cc)
-                progress.session_chunk_start(session, chunk=incomplete_cc_stack[0])
+                progress.session_chunk_start_exec(session, chunk=incomplete_cc_stack[0])
                 cc_jupyter_id = kernel_client.execute('\n'.join(icc.code_str for icc in incomplete_cc_stack))
             deadline = time.monotonic() + session.jupyter_timeout
             while True:
@@ -173,7 +202,10 @@ async def exec(session: Session, *, cache_key_path: pathlib.Path, progress: Prog
                             file_name = f'''{cc.options['name']}-{len(cc.rich_output)+1}.{file_extension}'''
                         session.files.append(file_name)
                         ro_path = cache_key_path / file_name
-                        ro_path.write_bytes(base64.b64decode(data))
+                        if file_extension == 'svg':
+                            ro_path.write_text(data, encoding='utf8')
+                        else:
+                            ro_path.write_bytes(base64.b64decode(data))
                         rich_output_files[mime_type] = ro_path.as_posix()
                     cc.rich_output.append(rich_output)
                     rich_output_text = kernel_msg_content['data'].get('text/plain')
@@ -200,9 +232,9 @@ async def exec(session: Session, *, cache_key_path: pathlib.Path, progress: Prog
                     kernel_has_errors = True
                     progress.session_chunk_stderr(session, chunk=cc, output=kernel_msg_text)
             if not incomplete_cc_stack:
-                progress.session_chunk_end(session, chunk=cc)
+                progress.session_chunk_end_exec(session, chunk=cc)
             else:
-                progress.session_chunk_end(session, chunk=incomplete_cc_stack[-1])
+                progress.session_chunk_end_exec(session, chunk=incomplete_cc_stack[-1])
                 incomplete_cc_stack = []
     finally:
         kernel_client.stop_channels()
